@@ -3,8 +3,9 @@
 from .indexed import Indexed
 from .ndprint import ndprint
 from .odict import ODict
-from ..utils.common import mstr, bstr, lls, exprstrs, findShape
+from ..utils.common import mstr, bstr, lls, wls, exprstrs, findShape
 from .dataset import Dataset, make_title_meta_l0, CompositeDataset
+from .shaped import Shaped
 from .indexed import Indexed
 
 try:
@@ -94,7 +95,7 @@ class TableModel():
 MdpInfo = Model['metadata']
 
 
-class TableDataset(CompositeDataset, TableModel):
+class TableDataset(CompositeDataset, TableModel, Shaped):
     """  Special dataset that contains a single Array Data object.
     A TableDataset is a tabular collection of Columns. It is optimized to work on array data..
     The column-wise approach is convenient in many cases. For example, one has an event list, and each algorithm is adding a new field to the events (i.e. a new column, for example a quality mask).
@@ -120,6 +121,7 @@ class TableDataset(CompositeDataset, TableModel):
 
     def __init__(self, data=None,
                  description=None,
+                 typ_=None,
                  version=None,
                  zInfo=None,
                  alwaysMeta=True,
@@ -132,7 +134,8 @@ class TableDataset(CompositeDataset, TableModel):
         # collect MDPs from args-turned-local-variables.
         metasToBeInstalled = OrderedDict(
             itertools.filterfalse(
-                lambda x: x[0] in ('self', '__class__', 'zInfo', 'kwds'),
+                lambda x: x[0] in ('self', '__class__', 'zInfo',
+                                   'kwds'),
                 locals().items())
         )
 
@@ -165,6 +168,7 @@ class TableDataset(CompositeDataset, TableModel):
 
         if data is None:
             super(TableDataset, self).setData(ODict())
+            self.updateShape()
             return
 
         current_data = self.getData()
@@ -191,6 +195,7 @@ class TableDataset(CompositeDataset, TableModel):
                                 '[[str, [], str]...], [[str, []]...], [[str, Column]...] needed.')
                     else:
                         # x is not string-started
+                        # e.g. [[1, 2, 3], [4, 5, 6]]
                         if current_data is None or len(current_data) <= ind:
                             # update the data of the ind-th column
                             self.setColumn('', Column(data=x, unit=None))
@@ -202,13 +207,15 @@ class TableDataset(CompositeDataset, TableModel):
                     raise ValueError(
                         'Cannot extract name and column from list member ' + str(x))
         elif issubclass(data.__class__, maplist):
+            # [Column, ... ]
             for k, v in data.items():
                 self.setColumn(k, v)
         else:
             raise TypeError('must be a Sequence or a Mapping. ' +
                             data.__class__.__name__ + ' found.')
+        self.updateShape()
 
-    def addColumn(self, name, column, col_des=False):
+    def addColumn(self, name, column, col_des=True):
         """ Adds the specified column to this table, and attaches a name
         to it. If the name is null, a dummy name "column"+column_count+1 is created, such that it can be accessed by getColumn(str).
 
@@ -217,7 +224,7 @@ class TableDataset(CompositeDataset, TableModel):
         Parameters:
         name - column name.
         column - column to be added.
-        col_des - if True (default) and column description is 'UNKNOWN', set to column name.
+        col_des - if True (default) and if column description is 'UNKNOWN' or `None`, set to column name.
         """
 
         d = self.getData()
@@ -233,10 +240,13 @@ class TableDataset(CompositeDataset, TableModel):
                 self._list[self.indexOf(name)] = column.getData()
             except ValueError as e:
                 self._list.append(column.getData())
-        if col_des and column.getDescription() == 'UNKNOWN':
+        des = column.getDescription()
+        if col_des and des == 'UNKNOWN' or des is None:
             column.setDescription(name)
         d[name] = column
 
+        self.updateShape()
+
     def removeColumn(self, key):
         """ Removes the columns specified by ``key``.
 
@@ -246,6 +256,7 @@ class TableDataset(CompositeDataset, TableModel):
         for name in self.getColumnMap(key).keys():
             self._list.pop(self.indexOf(name))
             del(self.data[name])
+        self.updateShape()
 
     def indexOf(self, key):
         """ Returns the index of specified column.
@@ -265,6 +276,7 @@ class TableDataset(CompositeDataset, TableModel):
         else:
             raise "key must be string or Column, not %s." % type(key).__name__
 
+        self.updateShape()
         return ks.index(k)
 
     def addRow(self, row, rows=False):
@@ -284,6 +296,9 @@ class TableDataset(CompositeDataset, TableModel):
                 d[c].data.extend(row[c])
             else:
                 d[c].data.append(row[c])
+            if hasattr(d[c], 'updateShape'):
+                d[c].updateShape()
+        self.updateShape()
 
     def getRowMap(self, rowIndex):
         """ Returns a dict of column-names as the keys and the objects located at a particular row(s) as the values.
@@ -355,12 +370,15 @@ class TableDataset(CompositeDataset, TableModel):
             for x in self.getData().values():
                 ret.append(x.data[rowIndex])
                 del x.data[rowIndex]
-                x.shape = findShape(x)
+                x.updateShape()
+            self.updateShape()
             return ret
         r = []
         for x in self.getData().values():
             r.append(x.data.pop(rowIndex))
-            x.shape = findShape(x)
+            x.updateShape()
+
+        self.updateShape()
         return r
 
     @ property
@@ -469,6 +487,8 @@ Default is to return all columns.
                 key = ''
         self.addColumn(name=key, column=value)
 
+        self.updateShape()
+
     def __iter__(self):
         for x in self.getData():
             yield x
@@ -512,10 +532,11 @@ Default is to return all columns.
     def toString(self, level=0,
                  tablefmt='grid', tablefmt1='simple', tablefmt2='simple',
                  width=0, param_widths=None, matprint=None, trans=True,
-                 heavy=True, center=-1, **kwds):
+                 heavy=True, center=-1, col_width=10, **kwds):
         """
         tablefmt2: format of 2D data, others see `MetaData.toString`.
         """
+
         if matprint is None:
             matprint = ndprint
 
@@ -548,17 +569,21 @@ Default is to return all columns.
 
         coldata = [list(itertools.islice(x.data, stp)) for x in cols]
 
-        nmun = zip((str(x) for x in self.getData().keys()),
+        # [(column name, unit), ...]. Widths of column head is limited
+        nmun = zip((wls(x, width=col_width) for x in self.getData().keys()),
                    (str(x.unit) for x in cols))
         hdr = list('%s\n(%s)' % nu for nu in nmun)
         d += matprint(coldata, trans=trans, headers=hdr,
                       tablefmt=tablefmt, tablefmt1=tablefmt1,
-                      tablefmt2=tablefmt2, center=center, **kwds)
+                      tablefmt2=tablefmt2, center=center,
+                      maxElem=sys.maxsize, **kwds)
         collen = self.getRowCount()
         if level and stp < collen:
             d += '(Only display %d rows of %d for level=%d.)' % (stp, collen, level)
         return f'{s}\n{d}{last}\n'
 
+    string = toString
+
     def __getstate__(self):
         """ Can be encoded with serializableEncoder """
         return OrderedDict(
