@@ -10,15 +10,20 @@ from .route.httppool_server import data_api, checkpath
 
 from .._version import __version__
 from ..utils import getconfig
-from ..utils.common import getUidGid
+from ..utils.common import getUidGid, trbk
 from ..pal.poolmanager import PoolManager as PM, DEFAULT_MEM_POOL
 
 from flasgger import Swagger
-from flask import Flask
+from werkzeug.exceptions import HTTPException
+from flask import Flask, make_response, jsonify
+from werkzeug.routing import RequestRedirect
+from werkzeug.routing import RoutingException, Map
 
 import builtins
 from collections import ChainMap
 import sys
+import json
+import time
 import os
 
 # sys.path.insert(0, abspath(join(join(dirname(__file__), '..'), '..')))
@@ -111,8 +116,9 @@ def init_httppool_server(app):
     poolpath_base = os.path.join(_basepath, pc['api_version'])
 
     if checkpath(poolpath_base, pc['serveruser']) is None:
-        app.logger.error('Store path %s unavailable.' % poolpath_base)
-        sys.exit(-2)
+        msg = 'Store path %s unavailable.' % poolpath_base
+        app.logger.error(msg)
+        return None
 
     app.config['POOLSCHEME'] = scheme
     app.config['POOLPATH_BASE'] = poolpath_base
@@ -128,23 +134,30 @@ def create_app(config_object=None, logger=None):
 
     if logger is None:
         logging = setup_logging()
-        logging.getLogger()
-        # logger = globals()['logger']
         logger = logging.getLogger()
 
+    config_object = config_object if config_object else getconfig.getConfig()
+    logger.setLevel(int(config_object['logginglevel']))
+
     app = Flask(__name__, instance_relative_config=True)
     app.config['SWAGGER'] = {
         'title': 'FDI %s HTTPpool Server' % __version__,
         'universion': 3,
         'openapi': '3.0.3',
+        'specs_route': config_object['api_base'] + '/apidocs/'
     }
+    swag['servers'].insert(0, {
+        'description': 'As in config file and server command line.',
+        'url': config_object['scheme']+'://' +
+        config_object['node']['host'] + ':' +
+        str(config_object['node']['port']) +
+        config_object['baseurl']
+    })
     swagger = Swagger(app, config=swag, merge=True)
-    #swagger = Swagger(app)
-
-    config_object = config_object if config_object else getconfig.getConfig()
+    # swagger.config['specs'][0]['route'] = config_object['api_base'] + s1
     app.config['PC'] = config_object
     app.config['LOGGER_LEVEL'] = logger.getEffectiveLevel()
-    #logging = setup_logging()
+    # logging = setup_logging()
     with app.app_context():
         init_httppool_server(app)
     # initialize_extensions(app)
@@ -152,5 +165,44 @@ def create_app(config_object=None, logger=None):
 
     app.register_blueprint(pools_api, url_prefix=config_object['baseurl'])
     app.register_blueprint(data_api, url_prefix=config_object['baseurl'])
+    addHandlers(app)
+    #app.url_map.strict_slashes = False
 
     return app
+
+
+def addHandlers(app):
+
+    # @app.errorhandler(RequestRedirect)
+    # def handle_redirect(error):
+    #     __import__('pdb').set_trace()
+
+    #     spec = 'redirect'
+
+    @app.errorhandler(Exception)
+    def handle_excep(error):
+        """ ref flask docs """
+        ts = time.time()
+        if issubclass(error.__class__, HTTPException):
+            if error.code == 409:
+                spec = "Conflict or updating. "
+            elif error.code == 500 and error.original_exception:
+                error = error.original_exception
+            else:
+                spec = ''
+            response = error.get_response()
+            t = ' Traceback: ' + trbk(error)
+            msg = '%s%d. %s, %s\n%s' % \
+                (spec, error.code, error.name, error.description, t)
+        elif issubclass(error.__class__, Exception):
+            response = make_response()
+            t = 'Traceback: ' + trbk(error)
+            msg = '%s. %s.\n%s' % (error.__class__.__name__,
+                                   str(error), t)
+        else:
+            response = make_response('', error)
+            msg = ''
+        w = {'result': 'FAILED', 'msg': msg, 'time': ts}
+        response.data = json.dumps(w)
+        response.content_type = 'application/json'
+        return response
