@@ -5,7 +5,7 @@ from ..model.user import getUsers, auth
 from ...utils.common import lls
 from ...dataset.deserialize import deserialize, deserialize_args
 from ...dataset.serializable import serialize
-from ...dataset.arraydataset import MediaWrapper
+from ...dataset.mediawrapper import MediaWrapper
 from ...pal.urn import makeUrn
 from ...dataset.classes import Classes
 from ...utils.common import trbk, getUidGid
@@ -22,6 +22,7 @@ from flasgger import swag_from
 from flask import request, make_response, jsonify, Blueprint, current_app
 from flask.wrappers import Response
 
+
 import sys
 import os
 import time
@@ -80,6 +81,7 @@ def checkpath(path, un):
 
     # logger.info('Setting owner, group, and mode...')
     if not setOwnerMode(p, un):
+        logger.info('Cannot set owner %s to %s.' % (un, str(p)))
         return None
 
     logger.debug('checked path at ' + str(p))
@@ -91,7 +93,7 @@ def checkpath(path, un):
 # @data_api.before_app_first_request
 
 
-def resp(code, result, msg, ts, serialize_out=False, ctype='application/json', length=70, req_auth=False):
+def resp(code, result, msg, ts, serialize_out=False, ctype='application/json', length=80, req_auth=False):
     """
     Make response.
 
@@ -105,12 +107,10 @@ def resp(code, result, msg, ts, serialize_out=False, ctype='application/json', l
         if serialize_out:
             # result is already in serialized form
             p = 'no-serialization-result-place-holder'
-            t = serialize({"code": code, "result": p,
-                           "msg": msg, "time": ts})
+            t = serialize({"result": p, "msg": msg, "time": ts})
             w = t.replace('"'+p+'"', result)
         else:
-            w = serialize({"code": code, "result": result,
-                           "msg": msg, "time": ts})
+            w = serialize({"result": result, "msg": msg, "time": ts})
     else:
         w = result
 
@@ -202,11 +202,21 @@ def delete_urn(parts):
 
 
 def parts2paths(parts):
-    colo = parts.replace('/', ':')
-    sp1 = colo.split(':')
-    if sp1[0].lower() == 'urn' or len(sp1[0]) == 0:
-        # ignore the 0th seg which is 'urn' or ''
-        paths = sp1[1:]
+    # parts == urn:{type}:{index}/...
+    # parts == /urn:{type}:{index}/...
+    # parts == :{type}:{index}/...
+    # parts == /{type}:{index}/...
+    # parts == /{type}/{index}/...
+    if parts[:4].lower() == 'urn:':
+        parts = parts[4:]
+    elif parts[:5].lower() == '/urn:':
+        parts = parts[5:]
+    elif parts[0] == '/' or parts[0] == ':':
+        parts = parts[1:]
+    # deal with the possible ':' before index
+    sp1 = parts.split('/')
+    if ':' in sp1[0]:
+        paths = sp1[0].split(':') + sp1[1:]
     else:
         paths = sp1
     return paths
@@ -249,6 +259,7 @@ def delete_product(paths, serialize_out=False):
 ######################################
 
 
+# @ data_api.route('/<string:pool>', methods=['POST'])
 @ data_api.route('/<string:pool>/', methods=['POST'])
 @ auth.login_required(role='read_write')
 def save_data(pool):
@@ -358,7 +369,7 @@ def data_paths(pool, data_paths):
     # do not deserialize if set True. save directly to disk
     serial_through = True
 
-    paths = [pool] + data_paths.replace(':', '/').split('/')
+    paths = [pool] + parts2paths(data_paths)
 
     logger.debug('*** method= %s pool= %s data_paths= %s paths= %s' %
                  (request.method, pool, str(data_paths), str(paths)))
@@ -421,7 +432,7 @@ def get_component_or_method(paths, mInfo, serialize_out=False):
     ts = time.time()
     logger.debug('get compo or meth: ' + str(paths))
     lp = len(paths)
-    # if paths[-1] == 'toString':
+    # if paths[-1] in ('toString', 'string'):
     #    __import__('pdb').set_trace()
 
     if paths[-1] == '':
@@ -432,7 +443,7 @@ def get_component_or_method(paths, mInfo, serialize_out=False):
             return 0, resp(200, ls, 'Getting %s members/attrbutes OK' % (path_str),
                            ts, serialize_out=False), 0
         else:
-            return 400, FAILED, '%s: %s' % (cmd, path_str)
+            return 400, FAILED, '%s' % (path_str)
     elif lp == 3:
         # url ends with index
         # no cmd, ex: test/fdi.dataset.Product/4
@@ -448,7 +459,8 @@ def get_component_or_method(paths, mInfo, serialize_out=False):
             code = 200
             msg = f'Getting {path_str} OK'
             compo_meth_name = path_str.split('.')[-1]
-            if compo_meth_name.startswith('toString'):
+            if compo_meth_name[:8] == 'toString' or \
+               compo_meth_name[:6] == 'string':
                 if 'html' in compo_meth_name:
                     ct = 'text/html'
                 elif 'fancy_grid' in compo_meth_name:
@@ -542,6 +554,7 @@ def setOwnerMode(p, username):
 
     uid, gid = getUidGid(username)
     if uid == -1 or gid == -1:
+        logger.debug(f'user {username} uid={uid} gid{gid}')
         return None
     try:
         os.chown(str(p), uid, gid)
@@ -570,34 +583,3 @@ def mkv(v, t):
     m = v if t == 'str' else None if t == 'NoneType' else Builtins[t](
         v) if t in Builtins else deserialize(v)
     return m
-
-
-@data_api.errorhandler(400)
-def bad_request(error):
-    ts = time.time()
-    w = {'result': 'FAILED', 'msg': 'Bad request. ' + str(error), 'time': ts}
-    return make_response(jsonify(w), 400)
-
-
-@data_api.errorhandler(401)
-def unauthorized(error):
-    ts = time.time()
-    w = {'result': 'FAILED',
-         'msg': 'Unauthorized. Authentication needed to modify. ' + str(error),
-         'time': ts}
-    return make_response(jsonify(w), 401)
-
-
-@data_api.errorhandler(404)
-def not_found(error):
-    ts = time.time()
-    w = {'result': 'FAILED', 'msg': 'Not found. ' + str(error), 'time': ts}
-    return make_response(jsonify(w), 404)
-
-
-@data_api.errorhandler(409)
-def conflict(error):
-    ts = time.time()
-    w = {'result': 'FAILED', 'msg': 'Conflict. Updating. ' + str(error),
-         'time': ts}
-    return make_response(jsonify(w), 409)
