@@ -57,7 +57,7 @@ class DictHk(Taggable):
         """
         Remove the given key.
         """
-        vals = themap.pop(key)
+        vals = themap.pop(key, None)
         # remove all items whose v is key in the otherosit map
         for val in vals[othername]:
             othermap[val][thename].remove(key)
@@ -101,3 +101,11 @@ class DictHk(Taggable):
         Tests if a tag exists.
         """
         return tag in self._tags
+
+    def getUrn(self, tag):
+        """
+        Gets the URNs corresponding to the given tag. Returns an empty list if tag does not exist.
+        """
+        if tag not in self._tags:
+            return []
+        return self._tags[tag]['urns']
