@@ -1,14 +1,16 @@
 # -*- coding: utf-8 -*-
 
-from .productpool import ManagedPool
+from .productpool import ManagedPool, PoolNotFoundError, MetaData_Json_Start, MetaData_Json_End
 from ..utils.common import pathjoin, trbk
+from .urn import makeUrn, Urn, parseUrn
+from ..dataset.deserialize import deserialize
 
 import filelock
+from functools import lru_cache
 import sys
 import shutil
 import mmap
 import time
-import json
 import os
 from os import path as op
 import logging
@@ -28,7 +30,9 @@ else:
 
 def wipeLocal(path):
     """
-    does the scheme-specific remove-all
+    does the scheme-specific remove-all.
+
+    A new directory at `path` will be created.
     """
     # logger.debug()
 
@@ -47,12 +51,16 @@ def wipeLocal(path):
 
 class LocalPool(ManagedPool):
     """ the pool will save all products in local computer.
+
+        :makenew: when the pool does not exist, make a new one (````True```; default) or throws `PoolNotFoundError` (```False```).
+
     """
 
-    def __init__(self, **kwds):
+    def __init__(self, makenew=True, **kwds):
         """ creates file structure if there isn't one. if there is, read and populate house-keeping records. create persistent files if not exist.
         """
         # print(__name__ + str(kwds))
+        self._makenew = makenew  # must preceed setup() in super
         super().__init__(**kwds)
 
     def setup(self):
@@ -66,7 +74,11 @@ class LocalPool(ManagedPool):
 
         real_poolpath = self.transformpath(self._poolname)
         if not op.exists(real_poolpath):
-            os.makedirs(real_poolpath)
+            if self._makenew:
+                os.makedirs(real_poolpath)
+            else:
+                raise PoolNotFoundError('poolname: %r poolurl: %r real_poolpath: %r' % (
+                    self._poolname, self._poolurl, real_poolpath))
         self._files = {}
         self._atimes = {}
         self._cached_files = {}
@@ -83,7 +95,7 @@ class LocalPool(ManagedPool):
         self.writeHK(fp0)
         return False
 
-    def readmmap(self, filename, close=False, check_time=False):
+    def readmmap(self, filename,  start=None, end=None, close=False, check_time=False):
         fp = op.abspath(filename)
         if check_time:
             sr = os.stat(fp)
@@ -91,27 +103,32 @@ class LocalPool(ManagedPool):
             # file hasnot changed since last time we read/wrote it.
             return None
         try:
-            if 1:  # fp not in self._files or self._files[fp] is None:
+            if 1:  # if fp not in self._files or self._files[fp] is None:
                 file_obj = open(fp, mode="r+", encoding="utf-8")
-                # with mmap.mmap(file_obj.fileno(), length=0, access=mmap.ACCESS_READ) as mmap_obj:
+                mmap_obj = mmap.mmap(
+                    file_obj.fileno(), length=0, access=mmap.ACCESS_READ)
+                fo = mmap_obj
             else:
-                file_obj = self._files[fp]
-                # file_obj.seek(0)
-            js = file_obj.read()
+                fo = self._files[fp]
+            if start is None:
+                js = fo.read()
+            else:
+                fo.seek(start)
+                js = fo.read(end - start)
         except Exception as e:
             msg = 'Error in HK reading. exc: %s trbk: %s.' % (str(e), trbk(e))
             logging.error(msg)
             raise NameError(msg)
         if 1:  # close:
-            file_obj.close()
+            fo.close()
             if fp in self._files:
                 del self._files[fp]
         else:
-            self._files[fp] = file_obj
+            self._files[fp] = fo
         if check_time:
             # save the mtime as the self atime
             self._atimes[fp] = sr.st_mtime_ns
-        return js
+        return js.decode('ascii')
 
     def readHK(self, hktype=None, serialize_out=False):
         """
@@ -156,16 +173,29 @@ class LocalPool(ManagedPool):
         else:
             return hk if hktype is None else hk[hktype]
 
-    def writeJsonmmap(self, fp, data, serialize_in=True, serialize_out=False, close=False, check_time=False, **kwds):
+    def writeJsonmmap(self, fp, data, serialize_in=True, serialize_out=False, close=False, check_time=False, meta_location=False, **kwds):
         """ write data in JSON from mmap file at fp.
 
         register the file. Leave file open by default `close`.
         data: to be serialized and saved.
         serialize_out: if True returns contents in serialized form.
+        :check_time: to check if file has not been written since we did last time. Default `False`.
+        :meta_location: return the start and end offsets of metadata in data JSON.  Default `False`.
+        :return:
+        int bytes written. If `meta_location` is ```True```, adding int int start and end point offsets of metadata in seriaized data.
         """
         from ..dataset.serializable import serialize
 
         js = serialize(data, **kwds) if serialize_in else data
+        #start = end = None
+        if meta_location:
+            # locate metadata
+            start = js.find(MetaData_Json_Start, 0)
+            # make end relative to file start
+            end = js.find(MetaData_Json_End, start) + start
+            start += len(MetaData_Json_Start)
+            end += len(MetaData_Json_End)
+
         fp = op.abspath(fp)
         if 1:  # fp not in self._files or self._files[fp] is None:
             file_obj = open(fp, mode="w+", encoding="utf-8")
@@ -193,7 +223,10 @@ class LocalPool(ManagedPool):
             self._cached_files[fp] = js
         l = len(js)
         logger.debug('JSON saved to: %s %d bytes' % (fp, l))
-        return l
+        if meta_location:
+            return l, start, end
+        else:
+            return l
 
     def writeHK(self, fp0=None):
         """ save the housekeeping data to disk
@@ -207,6 +240,60 @@ class LocalPool(ManagedPool):
                                     check_time=True)
         return l
 
+    def setMetaByUrn(self, start, end, urn):
+        """
+        Sets the location of the meta data of the specified data to the given URN.
+
+        :data: usually serialized Product.
+        """
+        u = urn.urn if issubclass(urn.__class__, Urn) else urn
+        if u not in self._urns:
+            raise ValueError(urn + ' not found in pool ' + self._poolname)
+        # char offset of the start and end points of metadata
+        if start >= 0 and end > 0:
+            mt = [start, end]
+        else:
+            mt = [None, None]
+        self._urns[u]['meta'] = mt
+
+    @ lru_cache(maxsize=1024)
+    def getMetaByUrnJson(self, urn, resourcetype, index):
+
+        # deserialize(prd[start+len(MetaData_Json_Start):end+len(MetaData_Json_End)])
+        try:
+            start, end = tuple(self._urns[urn]['meta'])
+        except KeyError as e:
+            msg = f"Trouble with {self._poolname}._urns[urn]['meta']"
+            logger.debug(msg)
+            raise e
+        js = self.schematicLoad(resourcetype=resourcetype,
+                                index=index, start=start, end=end,
+                                serialize_out=True)
+
+        return js
+
+    def getCacheInfo(self):
+        info = super().getCacheInfo()
+        for i in ['getMetaByUrnJson', 'transformpath']:
+            info[i] = getattr(self, i).cache_info()
+
+        return info
+
+    def getMetaByUrn(self, urn, resourcetype=None, index=None):
+        """ 
+        Get all of the meta data belonging to a product of a given URN.
+
+        mh: returns an iterator.
+        """
+        if urn is None:
+            return None  # TODO self._meta.keys()
+        #uobj = Urn(urn=urn)
+        if resourcetype is None or index is None:
+            poolname, resourcetype, index = parseUrn(urn)
+        m = self.getMetaByUrnJson(urn, resourcetype=resourcetype,
+                                  index=index)
+        return deserialize(m)  # self._urns[urn]['meta']
+
     def doSave(self, resourcetype, index, data, tag=None, serialize_in=True, **kwds):
         """
         does the media-specific saving.
@@ -217,17 +304,22 @@ class LocalPool(ManagedPool):
         fp = pathjoin(fp0, quote(resourcetype) + '_' + str(index))
         try:
             # t0 = time.time()
-            l = self.writeJsonmmap(
-                fp, data, serialize_in=serialize_in, close=True, **kwds)
+            l, start, end = self.writeJsonmmap(
+                fp, data, serialize_in=serialize_in, close=True,
+                meta_location=True, **kwds)
+
+            urn = makeUrn(self._poolname, resourcetype, index)
+            self.setMetaByUrn(start, end, urn)
+
             l += self.writeHK(fp0)
             # print('tl %.8f %9d' % (time.time()-t0, l))
             logger.debug('HK written')
         except IOError as e:
             logger.error('Save failed. exc: %s trbk: %s.' % (str(e), trbk(e)))
             raise e  # needed for undoing HK changes
-        return l
+        return l, start, end
 
-    def doLoad(self, resourcetype, index, serialize_out=False):
+    def doLoad(self, resourcetype, index, start=None, end=None, serialize_out=False):
         """
         does the action of loading.
 
@@ -237,7 +329,7 @@ class LocalPool(ManagedPool):
         indexstr = str(index)
         pp = self.transformpath(self._poolname) + '/' + \
             resourcetype + '_' + indexstr
-        js = self.readmmap(pp, close=True)
+        js = self.readmmap(pp, start=start, end=end, close=True)
         if serialize_out:
             r = js
         else:
@@ -280,7 +372,7 @@ class LocalPool(ManagedPool):
         self._classes.clear()
         self._tags.clear()
         self._urns.clear()
-
+        # will leave a newly made pool dir
         wipeLocal(self.transformpath(self._poolname))
         return 0
 
