@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
 import requests
+import functools
 import logging
 import sys
 from requests.auth import HTTPBasicAuth
@@ -29,6 +30,13 @@ logger = logging.getLogger(__name__)
 pcc = getConfig()
 defaulturl = 'http://' + pcc['node']['host'] + \
     ':' + str(pcc['node']['port']) + pcc['baseurl']
+AUTHUSER = pcc['node']['username']
+AUTHPASS = pcc['node']['password']
+
+
+@functools.lru_cache(maxsize=16)
+def getAuth(user=AUTHUSER, password=AUTHPASS):
+    return HTTPBasicAuth(user, password)
 
 
 def urn2fdiurl(urn, poolurl, contents='product', method='GET'):
@@ -94,7 +102,8 @@ def urn2fdiurl(urn, poolurl, contents='product', method='GET'):
         elif contents in ['classes', 'urns', 'tags']:
             ret = poolurl + '/hk/' + contents
         elif contents.split('__')[0] in WebAPI:
-            ret = poolurl + '/api/' + contents
+            # append a '/' for flask
+            ret = poolurl + '/api/' + contents + '/'
         else:
             raise ValueError(
                 'No such method and contents composition: ' + method + ' / ' + contents)
@@ -143,18 +152,15 @@ def save_to_server(data, urn, poolurl, tag, no_serial=False):
 ts into the pool
     no_serial: do not serialize the data.
     """
-    user = pcc['auth_user']
-    password = pcc['auth_pass']
-    auth = HTTPBasicAuth(user, password)
+    auth = getAuth()
     api = urn2fdiurl(urn, poolurl, contents='product', method='POST')
     # print('POST API: ' + api)
     headers = {'tags': tag}
     sd = data if no_serial else serialize(data)
     res = requests.post(
         api, auth=auth, data=sd, headers=headers)
-    result = deserialize(res.text)
-    # print(result)
-    return result
+    # print(res)
+    return res
 
 
 def read_from_server(urn, poolurl, contents='product'):
@@ -163,9 +169,7 @@ def read_from_server(urn, poolurl, contents='product'):
     urn: to extract poolname, product type, and index if any of these are needed
     poolurl: the only parameter must be provided
     """
-    user = pcc['auth_user']
-    password = pcc['auth_pass']
-    auth = HTTPBasicAuth(user, password)
+    auth = getAuth()
     api = urn2fdiurl(urn, poolurl, contents=contents)
     # print("GET REQUEST API: " + api)
     res = requests.get(api, auth=auth)
@@ -180,9 +184,7 @@ def put_on_server(urn, poolurl, contents='pool'):
     urn: to extract poolname, product type, and index if any of these are needed
     poolurl: the only parameter must be provided
     """
-    user = pcc['auth_user']
-    password = pcc['auth_pass']
-    auth = HTTPBasicAuth(user, password)
+    auth = getAuth()
     api = urn2fdiurl(urn, poolurl, contents=contents, method='PUT')
     # print("DELETE REQUEST API: " + api)
     res = requests.put(api, auth=auth)
@@ -196,9 +198,7 @@ def delete_from_server(urn, poolurl, contents='product'):
     urn: to extract poolname, product type, and index if any of these are needed
     poolurl: the only parameter must be provided
     """
-    user = pcc['auth_user']
-    password = pcc['auth_pass']
-    auth = HTTPBasicAuth(user, password)
+    auth = getAuth()
     api = urn2fdiurl(urn, poolurl, contents=contents, method='DELETE')
     # print("DELETE REQUEST API: " + api)
     res = requests.delete(api, auth=auth)
