@@ -2,15 +2,17 @@
 
 from .masked import masked
 from .ydump import ydump
+from .. import dataset
 
 import hashlib
 import array
 import traceback
 import pprint
+import copy
 import pwd
 import logging
 from itertools import zip_longest
-from collections.abc import Sequence
+from collections.abc import Sequence, Mapping
 import sys
 if sys.version_info[0] >= 3:  # + 0.1 * sys.version_info[1] >= 3.3:
     PY3 = True
@@ -109,8 +111,7 @@ def mstr(obj, level=0, width=1, excpt=None, indent=4, depth=0,
     if level == 0:
         if not hasattr(obj, 'items'):
             return bstr(obj, level=level, **kwds)
-        from fdi.dataset.metadata import MetaData
-        if issubclass(obj.__class__, MetaData):
+        if issubclass(obj.__class__, dataset.metadata.MetaData):
             return obj.toString(level=level,
                                 tablefmt=tablefmt, tablefmt1=tablefmt1,
                                 tablefmt2=tablefmt2,
@@ -138,8 +139,7 @@ def mstr(obj, level=0, width=1, excpt=None, indent=4, depth=0,
             # returns value of value if possible. limit to 40 char
             obj = obj.getValue() if hasattr(obj, 'getValue') else obj
             return bstr(obj, length=80, level=level, **kwds)
-        from fdi.dataset.metadata import MetaData
-        if issubclass(obj.__class__, MetaData):
+        if issubclass(obj.__class__, dataset.metadata.MetaData):
             return obj.toString(level=level, **kwds) + '\n'
         else:
             pat = '%s= {%s}' if depth == 0 else '%s= %s'
@@ -379,20 +379,40 @@ def attrstr1(p, v, missingval='', ftime=False, state=True, width=1, **kwds):
     return vs
 
 
-def exprstrs(param, v='_value', **kwds):
-    """ Generates a set of strings for toString().
+def exprstrs(param, v='_value', extra=False, **kwds):
+    """ Generates a set of strings for param.toString().
 
+    :param: Parameeter or xDstaset.
+    :extra: Whether to include less often used attributes suc as ```fits_keyword```.
     """
-
+    if issubclass(param.__class__, dataset.metadata.Parameter):
+        extra_attrs = copy.copy(param._all_attrs)
+    elif issubclass(param.__class__, (dataset.arraydataset.ArrayDataset,
+                                      dataset.tabledataset.TableDataset,
+                                      dataset.unstructureddataset.UnstrcturedDataset)):
+        extra_attrs = dict((n, v['default'])
+                           for n, v in param.zInfo['metadata'].items())
+    else:
+        extra_attrs = {}
     ts = attrstr(param, '_type', **kwds)
+    if 'typ_' in extra_attrs:
+        extra_attrs.pop('typ_', '')
+    else:  # Dataset
+        extra_attrs.pop('type', '')
     vs = attrstr(param, v, ftime=True, **kwds)
+    extra_attrs.pop('value', '')
     fs = attrstr(param, '_default', ftime=True, **kwds)
+    extra_attrs.pop('default', '')
     ds = attrstr(param, 'description', **kwds)
+    extra_attrs.pop('description')
     gs = attrstr(param, '_valid', ftime=True, **kwds)
+    extra_attrs.pop('valid', '')
     us = attrstr(param, '_unit', **kwds)
+    extra_attrs.pop('unit', '')
     cs = attrstr(param, '_typecode', **kwds)
+    extra_attrs.pop('typecode', '')
 
-    return (vs, us, ts, ds, fs, gs, cs)
+    return (vs, us, ts, ds, fs, gs, cs, extra_attrs)
 
 
 def pathjoin(*p):
@@ -490,6 +510,10 @@ def ld2tk(v):
     return y
 
 
+class UserOrGroupNotFoundError(BaseException):
+    pass
+
+
 def getUidGid(username):
     """ returns the UID and GID  of the named user.
 
@@ -503,6 +527,8 @@ def getUidGid(username):
             '. check config. ' + str(e) + trbk(e)
         logger.error(msg)
         uid = -1
+        # UserOrGroupNotFoundError(msg).with_traceback(sys.exc_info()[2])
+        raise
     # do if platform supports.
     try:
         gid = pwd.getpwnam(username).pw_gid
@@ -511,24 +537,28 @@ def getUidGid(username):
             '. check config. ' + str(e) + trbk(e)
         gid = -1
         logger.error(msg)
+        raise
 
     return uid, gid
 
 
-def findShape(data):
-    """
+def findShape(data, element_seq=(str)):
+    """ Shape of list/dict of list/dict.
+
+    :element_seq: treat elements of these sequence types as scalars.
     """
     if data is None:
         return None
     shape = []
     d = data
     while d is not None:
-        if issubclass(d.__class__, (str)):
+        if issubclass(d.__class__, element_seq):
             d = None
         else:
             try:
                 shape.append(len(d))
-                d = d[0]
+                d = list(d.values())[0] if issubclass(
+                    d.__class__, Mapping) else d[0]
             except (TypeError, IndexError, KeyError) as e:
                 d = None
     return tuple(shape)
