@@ -1,3 +1,6 @@
+# -*- coding: utf-8 -*-
+
+from functools import lru_cache
 import copy
 
 #      Dictionary of Commonly Used FITS Keywords
@@ -220,7 +223,123 @@ FITS_keywords_CLASS = {
     'DATAMAX': 'DATAMAX'
 }
 
+
+# Dictionary of commonly used HCSS keywords that follow a numbering pattern
+# e.g. OBSID123': 'obsid123, PROP1': 'proposal1
+FITS_keywords_Numbered = {
+    'OBSID': 'obsid',
+    'PROP': 'proposal',
+    'CUNIT': 'cunit',
+    'CONSNAM': 'constraintName',
+    'CONSTYP': 'constraintType',
+    'CONSINF': 'constraintInfo',
+    'CONS_ST': 'timeConstraintStart',
+    'CONS_EN': 'timeConstraintEnd',
+    'YARRAY': 'yArray',
+    'ZARRAY': 'zArray',
+    'PROCOI': 'propCoI',
+    'PERTNAM': 'perturberName',
+    'PERTGM': 'perturberGM',
+    'CALTBC': 'calTableComment',
+    # -- HCSS-19602 Deconvolution task',
+    'LOF': 'loFreq',
+    'NBD': 'num_bad_chan_in_scan',
+    'BSC': 'bad_scan',
+    'POBS_': 'photObsid',
+}
+
+
+FITS_keywords_2 = {
+    'APID': 'apid',
+    'CHANGLOG': 'changelog',
+    'DEC': 'dec',
+    'DEC_NOM': 'decNominal',
+    'DEC_OBJ': 'decObject',
+    'DELTAPIX': 'deltaPix',
+    'DESC': 'description',
+    'DETECTOR': 'arrayName',
+    'END_WL': 'endWavelength',
+    'ERROR': 'error',
+    'EXP_TEXT': 'explanatoryText',
+    'FINETIME': 'fineTime',
+    'FORMATV': 'formatVersion',
+    'OBJTYPE': 'objectType',
+    'OBS_ID': 'obsid',
+    'OBS_MODE': 'obsMode',
+    'OBSSTATE': 'obsState',
+    'ODNUMBER': 'odNumber',
+    'OFF_POS': 'offPosFlag',
+    'ONSRCTIM': 'onSourceTime',
+    'ONTARF': 'onTargetFlag',
+    'ORIGIN': 'origin',
+}
+
+FITS_KEYWORDS = copy.copy(FITS_keywords_Standard)
+FITS_KEYWORDS.update(FITS_keywords_HEASARC)
+FITS_KEYWORDS.update(FITS_keywords_CLASS)
+FITS_KEYWORDS.update(FITS_keywords_Numbered)
+FITS_KEYWORDS.update(FITS_keywords_2)
+
+Param_Names = dict((v, k) for k, v in FITS_KEYWORDS.items())
+
+
+@lru_cache(maxsize=256)
+def getFitsKw(name, ndigits=2, extra=None):
+    """ Returns the FITS keyword for a name.
+
+    If `name` ends with a digit, split `name` to a digital part consists if all digits on the right, and a "non-digital part" on the left. Take `ndigits` continuoud digits, counting from right, to form the "numeric-part". The "non-digital part", or the `name` if not endibg with digits, get the pre-translation according to:
+
+    1. Look up in the `Param_Names` table (inverse `FITS_KEYWORDS` table, if fails,
+    2. try the `extra` dictionary if provided. if fails,
+    3. take key value
+
+    If `name` ends with a digit, append the "numeric-part" to the first `8 - ndigits` (maximum) of characters from pre-transition, uppercased, to form the resukt;
+ else take a maximum of 8 characters from pre-transition. uppercased, to form the resukt.
+
+    :name: the name of e.g. a parameter.
+    :ndigits: how many digits (right to left) to take maximum if `name` ends with digits. default 2. Raises `ValueError` if more than 7.
+    :extra: tuple of `(fits,para)` tuples to provide more look-up dictionary.
+    :returns: FITS keyword.
+    """
+    if ndigits > 7:
+        raise ValueError(
+            'Cannot allow %d digits in FITS keywords (max 7).' % ndigits)
+    lname = len(name)
+    if extra is None:
+        pass
+    elif not issubclass(extra.__class__, (tuple)) or\
+            (extra and not issubclass(extra[0].__class__, tuple)):
+        raise TypeError(
+            '"extra" must be a tuple of a seriese (param:fitsKw) tuples.')
+    extradict = dict(extra) if extra else {}
+
+    non_digital = name.rstrip('0123456789')
+    lnondigi = len(non_digital)
+    endswith_digit = lname > lnondigi
+    if endswith_digit:
+        # has trailing digits
+        digital_part = name[lnondigi:]
+        numeric_part = digital_part[-ndigits:]
+        key = non_digital
+    else:
+        key = name
+    if key in Param_Names:
+        pre_translation = Param_Names[key]
+    elif key in extradict:
+        pre_translation = extradict[key]
+    else:
+        pre_translation = key
+    if endswith_digit:
+        return pre_translation[:8-ndigits].upper() + numeric_part
+    else:
+        return pre_translation[:8].upper()
+
+
+##########################
+######## NOT USED ########
+##########################
 FITS_keywords_HCSS = {
+
     # Dictionary of commonly used HCSS keywords
     #
     'ACMSMODE': 'acmsMode',
@@ -1299,30 +1418,3 @@ FITS_keywords_HCSS = {
     'COLMIN': 'colMin',
     'COLMAX': 'colMax',
 }
-
-# Dictionary of commonly used HCSS keywords that follow a numbering pattern
-# e.g. OBSID123': 'obsid123, PROP1': 'proposal1
-FITS_keywords_Numbered = {
-    'OBSID': 'obsid',
-    'PROP': 'proposal',
-    'CUNIT': 'cunit',
-    'CONSNAM': 'constraintName',
-    'CONSTYP': 'constraintType',
-    'CONSINF': 'constraintInfo',
-    'CONS_ST': 'timeConstraintStart',
-    'CONS_EN': 'timeConstraintEnd',
-    'YARRAY': 'yArray',
-    'ZARRAY': 'zArray',
-    'PROCOI': 'propCoI',
-    'PERTNAM': 'perturberName',
-    'PERTGM': 'perturberGM',
-    'CALTBC': 'calTableComment',
-    # -- HCSS-19602 Deconvolution task',
-    'LOF': 'loFreq',
-    'NBD': 'num_bad_chan_in_scan',
-    'BSC': 'bad_scan',
-    'POBS_': 'photObsid',
-}
-
-FITS_KEYWORDS = copy.copy(FITS_keywords_Standard)
-FITS_KEYWORDS.update(FITS_keywords_HEASARC)
