@@ -30,14 +30,15 @@ from fdi.dataset.messagequeue import MqttRelayListener, MqttRelaySender
 from fdi.dataset.composite import Composite
 from fdi.dataset.metadata import Parameter, MetaData, make_jsonable
 from fdi.dataset.metadataholder import MetaDataHolder
-from fdi.dataset.numericparameter import NumericParameter
+from fdi.dataset.numericparameter import NumericParameter, BooleanParameter
 from fdi.dataset.stringparameter import StringParameter
 from fdi.dataset.dateparameter import DateParameter
 from fdi.dataset.datatypes import DataTypes, DataTypeNames
 from fdi.dataset.attributable import Attributable
 from fdi.dataset.abstractcomposite import AbstractComposite
 from fdi.dataset.datawrapper import DataWrapper, DataWrapperMapper
-from fdi.dataset.arraydataset import ArrayDataset, Column, MediaWrapper
+from fdi.dataset.arraydataset import ArrayDataset, Column
+from fdi.dataset.mediawrapper import MediaWrapper
 from fdi.dataset.tabledataset import TableDataset
 from fdi.dataset.dataset import Dataset, CompositeDataset
 from fdi.dataset.indexed import Indexed
@@ -69,11 +70,15 @@ Classes.updateMapping()
 # make format output in /tmp/output.py
 mk_output = 0
 
+if mk_output:
+    with open('/tmp/output.py', 'wt') as f:
+        f.write('# -*- coding: utf-8 -*-\n')
+
 if __name__ == '__main__' and __package__ is None:
     # run by python3 tests/test_dataset.py
 
     if not mk_output:
-        from outputs import nds20, nds30, nds2, nds3, out_Dataset, out_ArrayDataset, out_TableDataset, out_CompositeDataset, out_FineTime
+        from outputs import nds20, nds30, nds2, nds3, out_Dataset, out_ArrayDataset, out_TableDataset, out_CompositeDataset, out_FineTime, out_MetaData
 else:
     # run by pytest
 
@@ -82,7 +87,7 @@ else:
     from pycontext import fdi
 
     if not mk_output:
-        from outputs import nds20, nds30, nds2, nds3, out_Dataset, out_ArrayDataset, out_TableDataset, out_CompositeDataset, out_FineTime
+        from outputs import nds20, nds30, nds2, nds3, out_Dataset, out_ArrayDataset, out_TableDataset, out_CompositeDataset, out_FineTime, out_MetaData
 
     import logging
     import logging.config
@@ -98,6 +103,10 @@ else:
     logger.debug('logging level %d' % (logger.getEffectiveLevel()))
     # logging.getLogger().setLevel(logging.DEBUG)
 
+    logging.getLogger("requests").setLevel(logging.WARN)
+    logging.getLogger("urllib3").setLevel(logging.WARN)
+    logging.getLogger("filelock").setLevel(logging.WARN)
+
 
 def checkgeneral(v):
     # can always add attributes
@@ -1007,6 +1016,29 @@ def test_NumericParameter():
     checkjson(v)
 
 
+def test_BooleanParameter():
+    v = BooleanParameter()
+    assert v.description == 'UNKNOWN'
+    assert v.value is None
+    assert v.type == 'boolean'
+    assert v.default is None
+    assert v.valid is None
+
+    a1 = 'a test BooleanParameter'
+    a2 = 100.234
+    a5 = True
+    a6 = (True, False)
+    v = BooleanParameter(description=a1, value=a2,
+                         default=a5, valid=a6)
+    assert v.description == a1
+    assert v.value == bool(a2)
+    assert v.type == 'boolean'
+    assert v.default == a5
+    assert v.valid == list(a6)
+
+    checkjson(v)
+
+
 def test_DateParameter():
 
     v = DateParameter()
@@ -1146,6 +1178,20 @@ def test_MetaData():
     v1['foo'] = Parameter('bar')
     assert v != v1
 
+    # toString wth extra attributes
+    ts = 'MetaData.toString with extra'
+    v = Product(ts)
+    v = v.meta
+    ts += '\n'
+    ts += v.toString(extra=True)
+    if mk_output:
+        print(ts)
+        with open('/tmp/output.py', 'a') as f:
+            clsn = 'out_MetaData'
+            f.write('%s = """%s"""\n' % (clsn, ts))
+    else:
+        assert ts == out_MetaData
+
     # listeners
     class MockMetaListener(MetaDataListener):
         pass
@@ -1287,7 +1333,7 @@ def test_Dataset():
     ts += v.toString(2)
     if mk_output:
         print(ts)
-        with open('/tmp/output.py', 'wt') as f:
+        with open('/tmp/output.py', 'a') as f:
             clsn = 'out_Dataset'
             f.write('%s = """%s"""\n' % (clsn, ts))
     else:
@@ -1317,22 +1363,26 @@ def do_ArrayDataset_init(atype):
     a6 = 'f'                  # typecode
     a7 = (8, 9)
     v = ArrayDataset(data=a1, unit=a2, description=a3,
-                     typ_=a4, shape=a7, typecode=a6)
+                     typ_=a4, typecode=a6)
     assert v.data == a1
     assert v.unit == a2
     assert v.description == a3
     assert v.typecode == a6
+    v.updateShape()
     assert v.shape == (len(a1),)
     v = ArrayDataset(data=a1)
     assert v.data == a1
     assert v.unit is None
     assert v.description == 'UNKNOWN'
     assert v.typecode == 'UNKNOWN'
+    assert v.updateShape()
     assert v.shape == (len(a1),)
 
     ashape = [[[1], [2]], [[3], [4]], [[5], [6]]]
     v2 = ArrayDataset(data=ashape)
     assert v2.shape == (3, 2, 1)
+    v2.pop()
+    assert v2.shape == (2, 2, 1)
     v2 = ArrayDataset(data=['poi'])
     assert v2.shape == (1,)
 
@@ -1351,12 +1401,12 @@ def do_ArrayDataset_init(atype):
 def check_MDP(x):
     for n, p in standardtestmeta().items():
         x.meta[n] = p
-    # when ``alwaysMeta`` is set, a Parameeter type attrbute becomes a Parameeter
+    # when ``alwaysMeta`` is set, a Parameter type attrbute becomes a Parameter
     assert x.alwaysMeta
     x.added_parameter = NumericParameter(42, description='A non-builtin param')
     assert x.meta['added_parameter'].value == 42
     assert issubclass(x.meta['added_parameter'].__class__, NumericParameter)
-    # wont work if not a Parameeter type
+    # wont work if not a Parameter type
     x.added_attribute = 111
     assert 'added_attribute' not in x.meta
     assert issubclass(x.added_attribute.__class__, int)
@@ -1497,6 +1547,7 @@ def do_ArrayDataset_func(atype):
 
     check_MDP(x)
     ts = '\n\nlevel 0\n'
+
     ts += x.toString()
     i = ts.index('0  0  0')
     if mk_output:
@@ -1617,6 +1668,7 @@ def test_TableDataset_func_row():
     c33, c44 = 3.3, 4.4
     v.addRow({'col4': c44, 'col3': c33})
     assert v.rowCount == 3
+
     cc.append(c33)
     assert v['col3'] == cc   # [1, 4, 3.3]
     # add rows
@@ -1624,12 +1676,14 @@ def test_TableDataset_func_row():
     v.addRow({'col4': c44, 'col3': c33}, rows=True)
     assert v.rowCount == 5
     cc.data.extend(c33)
+    # the above was done on built-in class so this has to be ddone manually
+    cc.updateShape()
+
     assert v['col3'] == cc
     # remove Row and rows
     assert v.removeRow(1) == [c1.data[1], c2.data[1]]
     assert v.rowCount == 4
     assert cc.pop(1) == c1.data[1]
-    cc.updateShape()  # not done automatically
     assert v['col3'] == cc
     # read rowa with slice
     s = v.getRow(slice(1, 3))
@@ -1692,14 +1746,18 @@ def test_TableDataset_func_column():
     v = TableDataset(data=a34)
     u = copy.deepcopy(a34)
 
-    assert v.getColumn(3) == Column(u[3])
-    assert v.getColumn('column4') == Column(u[3])
-    assert v.getColumn([1, 2]) == [Column(u[1]), Column(u[2])]
-    assert v.getColumn(['column2', 'column3']) == [Column(u[1]), Column(u[2])]
-    assert v.getColumn([False, True, False, False]) == [Column(u[1])]
-    assert v.getColumn([True, False]) == [Column(u[0])]
-    assert v.getColumn([True, False, False, False, False]) == [Column(u[0])]
-    assert v.getColumn([True, False, False, False, True]) == [Column(u[0])]
+    cu0 = Column(u[0], description='column1')
+    cu1 = Column(u[1], description='column2')
+    cu2 = Column(u[2], description='column3')
+    cu3 = Column(u[3], description='column4')
+    assert v.getColumn(3) == cu3
+    assert v.getColumn('column4') == cu3
+    assert v.getColumn([1, 2]) == [cu1, cu2]
+    assert v.getColumn(['column2', 'column3']) == [cu1, cu2]
+    assert v.getColumn([False, True, False, False]) == [cu1]
+    assert v.getColumn([True, False]) == [cu0]
+    assert v.getColumn([True, False, False, False, False]) == [cu0]
+    assert v.getColumn([True, False, False, False, True]) == [cu0]
 
     # slice
     sliced = v[1:3]   # a list if the 2nd and 3rd cols
@@ -1965,6 +2023,8 @@ def demo_TableDataset():
     print(table["Time"].data)
     print(table["Time"].unit)
 
+    return table
+
 
 def test_CompositeDataset_init():
     # constructor
@@ -2060,11 +2120,13 @@ def test_CompositeDataset_init():
     v3.set(a10, x)
     v3.meta[a11] = a12
     ts = 'level 0\n'
-    ts += v3.toString()
+    s3 = v3.toString()
+    ts += s3
     ts += 'level 1, repr\n'
     ts += v3.toString(1)
     ts += 'level 2,\n'
     ts += v3.toString(2)
+    assert v3.string() == s3
     if mk_output:
         print(ts)
         with open('/tmp/output.py', 'a') as f:
@@ -2108,6 +2170,7 @@ def test_UnstrcturedDataset():
     assert v == u.data["results"]["Time_Energy_Pos"]["Energy"]["_ATTR_data"]
     assert s == '.data["results"]["Time_Energy_Pos"]["Energy"]["_ATTR_data"]'
 
+    print('cache:', fdi.dataset.unstructureddataset.getCacheInfo())
     checkjson(u)
     checkgeneral(u)
 
@@ -2302,6 +2365,7 @@ def test_jsonPath():
                  ('store/bicycle/color', 'red'),
                  ('store/bicycle/price', 19.95)
                  ]
+    print('cache:', fdi.dataset.unstructureddataset.getCacheInfo())
 
 
 def test_Indexed():
@@ -2460,6 +2524,8 @@ def demo_CompositeDataset():
     print(child.__class__)
     print(child)
 
+    return c
+
 
 def test_FineTime():
     # default
